from __future__ import print_function
# import jmp_model as jmp
import jmp_model_bootstrap_forest as jmp
import json
import traceback

import os
import ctypes

if os.name == "posix":
    for d, _, files in os.walk('lib'):
        for f in files:
            if f.endswith('.a'):
                continue
            ctypes.cdll.LoadLibrary(os.path.join(d, f))


def handle(event, context):
    try:
        print("event: " + json.dumps(event, indent=2))
        method = event['httpMethod']
        if method == "GET":
                input_data = event["queryStringParameters"]
        elif method == "POST":
            if event["body"]:
                input_data = json.loads(event["body"])
            else:
                input_data = {}
        else:
            return error("Unsupported HTTP method '%s'" % method)
        route = event['resource'].split("/")[-1]
        result = "undefined"
        if route == "predict":
            if isinstance(input_data, list):
                result = score_chunk(input_data)
            elif isinstance(input_data, dict):
                result = score_row(input_data)
            else:
                return error("Unsupported input type: '%s'" % input_data)
        elif route == "metadata":
            result = metadata()
        elif route == "numpy":
            result = test_numpy()
        else:
            return error("Unknown route: '%s'" % route)
        return success(result)
    except Exception as e:
        return error(e)


def metadata():
    mm = {}
    mm["model"] = jmp.getModelMetadata()
    mm["input"] = jmp.getInputMetadata()
    mm["output"] = jmp.getOutputMetadata()
    return mm


def score_chunk(input_chunk):
    result = []
    if (len(input_chunk) == 0):
        return result
    result_key = get_result_key(input_chunk[0])
    for row in input_chunk:
        row[result_key] = score_row(row)
        result.append(row)
    return result


def score_row(input_row):
    input_mm = jmp.getInputMetadata()
    for input_key, input_type in input_mm.items():
        # decode? "Petal+width" => "Petal width"
        if input_key in input_row.keys():
            event_value = input_row[input_key]
            # could check or convert type as well
            if input_type == 'str':
                input_row[input_key] = event_value
            else:
                assert(input_type == 'float')
                if (event_value == ""):
                    input_row[input_key] = float('nan')
                else:
                    input_row[input_key] = float(event_value)
        else:
            # missing argument
            raise Exception("Missing '%s' in '%s'" % (input_key,
                            json.dumps(input_row)))
    out = {}
    # sending back just the final scoring result
    # could also send individual probabilities, stored in out
    result = jmp.score(input_row, out)
    return result


def get_result_key(input_chunk):
    model_mm = jmp.getModelMetadata()
    if "predicted" in model_mm and model_mm["predicted"]:
        base_result_key = "predicted_" + model_mm["predicted"]
    else:
        base_result_key = "predicted_result"
    result_key = base_result_key
    i = 1
    while result_key in input_chunk.keys() and i < 1000:
        result_key = "%s_%d" % (base_result_key, i)
    if i == 1000:
        raise Exception("Failed to create result key")
    return result_key


# def test_numpy():
#     print("test_numpy")
#     import numpy as np
#     a = np.array([[1.0, 2.0], [3.0, 4.0]])
#     a.transpose()
#     np.linalg.inv(a)
#     np.dot(a, a)
#     np.linalg.eig(a)
#     result = np.linalg.det(a)
#     print(result)
#     return result
#


def success(result):
    return {
        "statusCode": 200,
        "headers": {
            "Access-Control-Allow-Origin": "*",  # CORS
            "Access-Control-Allow-Credentials": True  # cookies, HTTPS
        },
        "body": json.dumps(result),
    }


def error(msg):
    just_the_string = ""
    try:
        raise Exception(msg)
    except Exception:
        just_the_string = traceback.format_exc()

    return {
        "statusCode": 500,
        "headers": {
            "Access-Control-Allow-Origin": "*",  # CORS
            "Access-Control-Allow-Credentials": True  # cookies, HTTPS
        },
        "body": json.dumps({
            "internalError": str(msg),
            "stackTrace": just_the_string
        })
    }
